import sys

import click

from mqer.timer import Timer
from mqer.utils import import_obj_string
from mqer.worker import Worker

__all__ = ('main',)


@click.group(invoke_without_command=True)
@click.pass_context
def mqer(ctx):
    pass


@mqer.command()
@click.argument("app")
@click.option("--pythonpath", default=".",  help='A directory to add to the Python path, e.g. "/home/pyprojects/myproject".')
@click.pass_context
def worker(ctx, pythonpath: str, app: str):
    """Start worker instance.

    For Examples:\n
    $ mqer worker autoscale_worker:app
    """
    if not pythonpath:
        pythonpath = "."
    sys.path.insert(0, pythonpath)
    app_instance = import_obj_string(app)
    if not isinstance(app_instance, Worker):
        click.echo("The app must be a subclass of mqer.Worker")
        ctx.exit()
    sys.exit(app_instance.start())


@mqer.command()
@click.argument("app")
@click.option("--pythonpath", default=".",  help='A directory to add to the Python path, e.g. "/home/pyprojects/myproject".')
@click.pass_context
def timer(ctx, pythonpath: str, app: str):
    """Start the timer periodic task scheduler.

    For Examples:\n
    $ mqer timer autoscale_worker:app
    """
    if not pythonpath:
        pythonpath = "."
    sys.path.insert(0, pythonpath)
    app_instance = import_obj_string(app)
    if not isinstance(app_instance, Timer):
        click.echo("The app must be a subclass of mqer.Timer")
        ctx.exit()
    sys.exit(app_instance.start())


def main():
    cli = click.CommandCollection(sources=[mqer])
    cli()