
import functools
from typing import Any, Callable

from django.conf import settings
from django.utils.deprecation import MiddlewareMixin

from mqer.jsonlogger import LogExtraContext, logextra
from mqer.utils import import_obj_string, make_msg_uuid


def get_ip(request) -> str:
    x_forwarded_for = request.META.get("HTTP_X_FORWARDED_FOR")
    if x_forwarded_for:
        ip = x_forwarded_for.split(",")[0]
    else:
        ip = request.META.get("REMOTE_ADDR", "")
    return ip


def get_username(user):
    try:
        username = user.get_username()
    except AttributeError:
        username = user.username

    return username


def _setup_logextra_from_request(request):
    tracer_id = request.META.get("HTTP_TRACER_ID", make_msg_uuid(to_hex=False))
    logextra.tracer_id = tracer_id
    if getattr(request, "user", None) and request.user.is_authenticated:
        logextra.visit_userid = request.user.pk
        logextra.visit_username = get_username(request.user)

    logextra.visit_user_ipaddress = get_ip(request)


def create_logextra_ctx(func: Callable[..., Any]):
    @functools.wraps(func)
    def do_logextra_ctx(request, *args, **kwargs):
        _logextra_setup_handler = getattr(settings, "LOGEXTRA_SETUP_HANDLER", _setup_logextra_from_request)
        if isinstance(_logextra_setup_handler, str):
            _logextra_setup_handler = import_obj_string(_logextra_setup_handler)

        with LogExtraContext():
            try:
                _logextra_setup_handler(request)
            except Exception:
                pass
            response = func(request,  *args, **kwargs)
            return response
    return do_logextra_ctx


class JSONLoggerExtraMiddleware(MiddlewareMixin):
    def __init__(self, get_response) -> None:
        _get_response = create_logextra_ctx(get_response)
        super().__init__(_get_response)
