import datetime
from decimal import Decimal
from importlib import import_module
from typing import Any
from uuid import uuid4


def import_obj_string(obj_path: str, sep: str = ".") -> Any:
    sep = ":" if ":" in obj_path else sep
    module_name, _, cls_name = obj_path.rpartition(sep)

    if not module_name:
        cls_name, module_name = '', cls_name
    try:
        module = import_module(module_name)
        return getattr(module, cls_name) if cls_name else module
    except AttributeError as err:
        raise ImportError('Module "%s" does not define a "%s" attribute/class' % (
            module_name, cls_name)
        ) from err


def make_msg_uuid(prefx: str = "", to_hex: bool = True):
    if to_hex:
        msg_uuid = uuid4().hex
    else:
        msg_uuid = str(uuid4())

    return "%s%s" % (prefx, msg_uuid)


_PROTECTED_TYPES = (type(None), int, float, str, Decimal, datetime.datetime, datetime.date, datetime.time)


def is_protected_type(obj: Any):
    return isinstance(obj, _PROTECTED_TYPES)


def force_str(s: Any, encoding: str = 'utf-8', strings_only: bool = False, errors: str = 'strict'):
    """
    If strings_only is True, don't convert (some) non-string-like objects.
    """
    # Handle the common case first for performance reasons.
    if issubclass(type(s), str):
        return s

    if strings_only and is_protected_type(s):
        return s

    if isinstance(s, bytes):
        s = str(s, encoding, errors)
    else:
        s = str(s)

    return s
